@@ -9,23 +9,23 @@ from sahi.utils.file import increment_path
 try:
     import norfair
     from norfair import Tracker, Detection
+    from norfair.tracker import TrackedObject
     from norfair.metrics import PredictionsTextFile, InformationFile
 except ImportError:
     raise ImportError('Please run "pip install -U norfair" to install norfair first for MOT format handling.')
 
 
-class GroundTruthTextFile(PredictionsTextFile):
-    def __init__(self, save_path="."):
+class MotTextFile(PredictionsTextFile):
+    def __init__(self, save_dir: str = ".", save_name: str = "gt"):
 
-        predictions_folder = os.path.join(save_path, "gt")
-        if not os.path.exists(predictions_folder):
-            os.makedirs(predictions_folder)
+        if not os.path.exists(save_dir):
+            os.makedirs(save_dir)
 
-        self.out_file_name = os.path.join(predictions_folder, "gt" + ".txt")
+        self.out_file_name = os.path.join(save_dir, save_name + ".txt")
 
         self.frame_number = 1
 
-    def update(self, predictions, frame_number=None):
+    def update(self, predictions: List[TrackedObject], frame_number: int = None):
         if frame_number is None:
             frame_number = self.frame_number
         """
@@ -68,13 +68,15 @@ def euclidean_distance(detection, tracked_object):
 
 
 class MotAnnotation:
-    def __init__(self, bbox: List[int], score: Optional[float] = 1):
+    def __init__(self, bbox: List[int], track_id: Optional[int] = -1, score: Optional[float] = 1):
         """
         Args:
             bbox (List[int]): [x_min, y_min, width, height]
+            track_id: (Optional[int]): track id of the annotation
             score (Optional[float])
         """
         self.bbox = bbox
+        self.track_id = track_id
         self.score = score
 
 
@@ -116,16 +118,80 @@ class MotFrame:
             norfair_detections.append(Detection(points=points, scores=scores))
         return norfair_detections
 
+    def to_norfair_trackedobjects(self, track_points: str = "bbox"):
+        """
+        Args:
+            track_points (str): 'centroid' or 'bbox'. Defaults to 'bbox'.
+        """
+        tracker = Tracker(
+            distance_function=euclidean_distance,
+            distance_threshold=30,
+            detection_threshold=0,
+            hit_inertia_min=10,
+            hit_inertia_max=12,
+            point_transience=4,
+        )
+
+        tracked_object_list: List[TrackedObject] = []
+        # convert all detections to norfair detections
+        for annotation in self.annotation_list:
+            # ensure annotation.track_id is not None
+            assert annotation.track_id is not None, TypeError(
+                "to_norfair_trackedobjects() requires annotation.track_id to be set."
+            )
+            # calculate bbox points
+            xmin = annotation.bbox[0]
+            ymin = annotation.bbox[1]
+            xmax = annotation.bbox[0] + annotation.bbox[2]
+            ymax = annotation.bbox[1] + annotation.bbox[3]
+            track_id = annotation.track_id
+            scores = None
+            # calculate points as bbox or centroid
+            if track_points == "bbox":
+                points = np.array([[xmin, ymin], [xmax, ymax]])  # bbox
+                if annotation.score is not None:
+                    scores = np.array([annotation.score, annotation.score])
+
+            elif track_points == "centroid":
+                points = np.array([(xmin + xmax) / 2, (ymin + ymax) / 2])  # centroid
+                if annotation.score is not None:
+                    scores = np.array([annotation.score])
+            else:
+                ValueError("'track_points' should be one of ['centroid', 'bbox'].")
+            # create norfair formatted detection
+            detection = Detection(points=points, scores=scores)
+            # create trackedobject from norfair detection
+            tracked_object = TrackedObject(
+                detection,
+                tracker.hit_inertia_min,
+                tracker.hit_inertia_max,
+                tracker.initialization_delay,
+                tracker.detection_threshold,
+                period=1,
+                point_transience=tracker.point_transience,
+                filter_setup=tracker.filter_setup,
+            )
+            tracked_object.id = track_id
+            # append to tracked_object_list
+            tracked_object_list.append(tracked_object)
+        return tracked_object_list
+
 
 class MotVideo:
     def __init__(
-        self, export_dir: str = "runs/mot", track_points: str = "bbox", tracker_kwargs: Optional[Dict] = dict()
+        self,
+        name: str = "sequence_name",
+        frame_rate: Optional[int] = 30,
+        image_height: int = 720,
+        image_width: int = 1280,
+        tracker_kwargs: Optional[Dict] = dict(),
     ):
         """
         Args
-            export_dir (str): Folder directory that will contain gt/gt.txt and seqinfo.ini
-                For details: https://github.com/tryolabs/norfair/issues/42#issuecomment-819211873
-            track_points (str): Track detections based on 'centroid' or 'bbox'. Defaults to 'bbox'.
+            name (str): Name of the video file.
+            frame_rate (int): FPS of the video.
+            image_height (int): Frame height of the video.
+            image_width (int): Frame width of the video.
             tracker_kwargs (dict): a dict contains the tracker keys as below:
                 - max_distance_between_points (int)
                 - min_detection_threshold (float)
@@ -135,37 +201,31 @@ class MotVideo:
                 For details: https://github.com/tryolabs/norfair/tree/master/docs#arguments
         """
 
-        self.export_dir: str = str(increment_path(Path(export_dir), exist_ok=False))
-        self.track_points: str = track_points
+        self.name = name
+        self.frame_rate = frame_rate
+        self.image_height = image_height
+        self.image_width = image_width
+        self.tracker_kwargs = tracker_kwargs
 
-        self.groundtruth_text_file: Optional[GroundTruthTextFile] = None
-        self.tracker: Optional[Tracker] = None
+        self.frame_list: List[MotFrame] = []
 
-        self._create_gt_file()
-        self._init_tracker(
-            tracker_kwargs.get("max_distance_between_points", 30),
-            tracker_kwargs.get("min_detection_threshold", 0),
-            tracker_kwargs.get("hit_inertia_min", 10),
-            tracker_kwargs.get("hit_inertia_max", 12),
-            tracker_kwargs.get("point_transience", 4),
-        )
-
-    def _create_info_file(self, seq_length: int):
+    def _create_info_file(self, seq_length: int, export_dir: str):
         """
         Args:
             seq_length (int): Number of frames present in video (seqLength parameter in seqinfo.ini)
                 For details: https://github.com/tryolabs/norfair/issues/42#issuecomment-819211873
+            export_dir (str): Folder directory that will contain exported file.
         """
         # set file path
-        filepath = Path(self.export_dir) / "seqinfo.ini"
+        filepath = Path(export_dir) / "seqinfo.ini"
         # create folder directory if not exists
         filepath.parent.mkdir(exist_ok=True)
         # create seqinfo.ini file with seqLength
         with open(str(filepath), "w") as file:
-            file.write(f"seqLength={seq_length}")
-
-    def _create_gt_file(self):
-        self.groundtruth_text_file = GroundTruthTextFile(save_path=self.export_dir)
+            file.write(f"seqLength={seq_length}\n")
+            file.write(f"frameRate={self.frame_rate}\n")
+            file.write(f"imWidth={self.image_width}\n")
+            file.write(f"imHeight={self.image_width}")
 
     def _init_tracker(
         self,
@@ -174,7 +234,7 @@ class MotVideo:
         hit_inertia_min: int = 10,
         hit_inertia_max: int = 12,
         point_transience: int = 4,
-    ):
+    ) -> Tracker:
         """
         Args
             max_distance_between_points (int)
@@ -182,9 +242,11 @@ class MotVideo:
             hit_inertia_min (int)
             hit_inertia_max (int)
             point_transience (int)
+        Returns:
+            tracker: norfair.tracking.Tracker
         For details: https://github.com/tryolabs/norfair/tree/master/docs#arguments
         """
-        self.tracker = Tracker(
+        tracker = Tracker(
             distance_function=euclidean_distance,
             initialization_delay=0,
             distance_threshold=max_distance_between_points,
@@ -193,10 +255,50 @@ class MotVideo:
             hit_inertia_max=hit_inertia_max,
             point_transience=point_transience,
         )
+        return tracker
 
     def add_frame(self, frame: MotFrame):
         assert type(frame) == MotFrame, "'frame' should be a MotFrame object."
-        norfair_detections: List[Detection] = frame.to_norfair_detections(track_points=self.track_points)
-        tracked_objects = self.tracker.update(detections=norfair_detections)
-        self.groundtruth_text_file.update(predictions=tracked_objects)
-        self._create_info_file(seq_length=self.groundtruth_text_file.frame_number)
+        self.frame_list.append(frame)
+
+    def export(self, export_dir: str = "runs/mot", type: str = "gt", use_tracker: bool = None, exist_ok=False):
+        """
+        Args
+            export_dir (str): Folder directory that will contain exported mot challenge formatted data.
+            type (str): Type of the MOT challenge export. 'gt' for groundturth data export, 'det' for detection data export.
+            use_tracker (bool): Determines whether to apply kalman based tracker over frame detections or not.
+                Default is True for type='gt'.
+                It is always False for type='det'.
+            exist_ok (bool): If True overwrites given directory.
+        """
+        assert type in ["gt", "det"], TypeError(f"'type' can be one of ['gt', 'det'], you provided: {type}")
+
+        export_dir: str = str(increment_path(Path(export_dir), exist_ok=exist_ok))
+
+        if type == "gt":
+            gt_dir = os.path.join(export_dir, self.name if self.name else "", "gt")
+            mot_text_file: MotTextFile = MotTextFile(save_dir=gt_dir, save_name="gt")
+            if not use_tracker:
+                use_tracker = True
+        elif type == "det":
+            det_dir = os.path.join(export_dir, self.name if self.name else "", "det")
+            mot_text_file: MotTextFile = MotTextFile(save_dir=det_dir, save_name="det")
+            use_tracker = False
+
+        tracker: Tracker = self._init_tracker(
+            self.tracker_kwargs.get("max_distance_between_points", 30),
+            self.tracker_kwargs.get("min_detection_threshold", 0),
+            self.tracker_kwargs.get("hit_inertia_min", 10),
+            self.tracker_kwargs.get("hit_inertia_max", 12),
+            self.tracker_kwargs.get("point_transience", 4),
+        )
+        for mot_frame in self.frame_list:
+            if use_tracker:
+                norfair_detections: List[Detection] = mot_frame.to_norfair_detections(track_points="bbox")
+                tracked_objects = tracker.update(detections=norfair_detections)
+            else:
+                tracked_objects = mot_frame.to_norfair_trackedobjects(track_points="bbox")
+            mot_text_file.update(predictions=tracked_objects)
+
+        info_dir = os.path.join(export_dir, self.name if self.name else "")
+        self._create_info_file(seq_length=mot_text_file.frame_number, export_dir=info_dir)
